@@ -1,4 +1,3 @@
-
 /*  ----------------------------------------------------------------------
 
     Copyright (C) 1998  Cesar Miquel  (miquel@df.uba.ar)
@@ -24,23 +23,13 @@
 
 #include "logview.h"
 
-#ifdef FIXME
-int ReadNPagesUp (Log * lg, Page * pg, int n);
-int ReadNPagesDown (Log * lg, Page * pg, int n);
-int ReadPageUp (Log * lg, Page * pg);
-#endif
-
-int isLogFile (char *filename);
-int isSameDay (time_t day1, time_t day2);
+int isLogFile (char *filename, gboolean show_error);
 int WasModified (Log *log);
-void reverse (char *line);
 void ParseLine (char *buff, LogLine * line);
-void MoveToMark ();
-void ReadLogStats (Log * log);
-void UpdateLogStats( Log *log );
 void CloseLog (Log * log);
 time_t GetDate (char *line);
 Log * OpenLogFile (char *filename);
-int ReadPageDown (Log *log, LogLine ***inp_mon_lines, gboolean exec_actions);
+gchar **ReadLastPage (Log *log);
+gchar **ReadNewLines (Log *log);
 
 #endif /* __LOGRTNS_H__ */
